unit uMain;

interface

uses
  SysUtils, Classes, Graphics, Controls, Forms, Menus,
  Dialogs, StdCtrls, Buttons, ExtCtrls, LResources, ComCtrls, uRedBlackTree, uRedBlackTreeNode;

type

  { TFrmMainWindow }

  TFrmMainWindow = class(TForm)
    Button4: TButton;
    Button5: TButton;
    Button6: TButton;
    Button7: TButton;
    cbStandardMode: TCheckBox;
    CheckBox3: TCheckBox;
    Edit1: TEdit;
    fdNodes: TFontDialog;
    fdNilLeaves: TFontDialog;
    GroupBox3: TGroupBox;
    Image1: TImage;
    GroupBox1: TGroupBox;
    ilIcons: TImageList;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    lblHeight: TLabel;
    lblFontNodes: TLabel;
    Label8: TLabel;
    lblFontNilLeaves: TLabel;
    lblBlackHeight: TLabel;
    lblWidth: TLabel;
    lblMinimum: TLabel;
    lblMaximum: TLabel;
    odTree: TOpenDialog;
    sdTree: TSaveDialog;
    Splitter1: TSplitter;
    ScrollBar1: TScrollBar;
    Label1: TLabel;
    Label2: TLabel;
    ScrollBar2: TScrollBar;
    Label3: TLabel;
    Label4: TLabel;
    GroupBox2: TGroupBox;
    Splitter2: TSplitter;
    Button1: TButton;
    cbDrawNilLeaves: TCheckBox;
    cbDrawBackground: TCheckBox;
    Button2: TButton;
    Splitter3: TSplitter;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    btnUndo: TToolButton;
    btnRedo: TToolButton;
    procedure cbDrawBackgroundChange(Sender: TObject);
    procedure cbDrawNilLeavesChange(Sender: TObject);
    procedure cbStandardModeChange(Sender: TObject);
    procedure CheckBox3Change(Sender: TObject);
    procedure Edit1Change(Sender: TObject);
    procedure Edit1KeyPress(Sender: TObject; var Key: char);
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure lblFontNodesClick(Sender: TObject);
    procedure lblFontNilLeavesClick(Sender: TObject);
    procedure ScrollBar1Change(Sender: TObject);
    procedure ScrollBar2Change(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure ToolButton1Click(Sender: TObject);
    procedure ToolButton2Click(Sender: TObject);
    procedure ToolButton3Click(Sender: TObject);
    procedure btnUndoClick(Sender: TObject);
    procedure btnRedoClick(Sender: TObject);

    procedure updateInfo(Sender: TObject);
    procedure handleInput(Sender: TObject);

    procedure traverseTree(Sender: TObject);

    procedure newElement();
  end;

  TFrmInfoWindow = class(TForm)
    Memo1: TMemo;
  end;

var
  FrmMainWindow: TFrmMainWindow;
  FrmInfoWindow: TFrmInfoWindow;
  strom: TRedBlackTree;

implementation

procedure TFrmMainWindow.newElement();
var
  e: Integer;
begin

  if (self.CheckBox3.Checked) then
    begin

      while True do
        begin
          e := Random(1000) + 1;

          if (strom.search(e) = NIL) then
            break;
        end;

      self.Edit1.Text := IntToStr(e);
    end
  else
    begin
      self.Edit1.Text := '';
      self.Edit1.SetFocus;
    end;
end;

procedure TFrmMainWindow.FormCreate(Sender: TObject);
begin

  strom := TRedBlackTree.Create(Image1, self.fdNodes.Font, self.fdNilLeaves.Font);

  Scrollbar1.Position := Scrollbar1.Max;
  Scrollbar2.Position := Scrollbar2.Max;

  cbDrawNilLeaves.Checked := strom.drawNilLeaves;
  cbDrawBackground.Checked := strom.drawBackgroundGradient;
  cbStandardMode.Checked := strom.standardMode;

  strom.OnChange := @self.updateInfo;
  strom.change();

  Randomize();
end;

procedure TFrmMainWindow.CheckBox3Change(Sender: TObject);
begin
  Edit1.Enabled := not Edit1.Enabled;
  newElement();
end;

procedure TFrmMainWindow.cbDrawBackgroundChange(Sender: TObject);
begin
  strom.drawBackgroundGradient := cbDrawBackground.Checked;
end;

procedure TFrmMainWindow.cbDrawNilLeavesChange(Sender: TObject);
begin
  strom.drawNilLeaves := cbDrawNilLeaves.Checked;
end;

procedure TFrmMainWindow.cbStandardModeChange(Sender: TObject);
begin
  strom.standardMode := cbStandardMode.Checked;
end;

procedure TFrmMainWindow.Edit1Change(Sender: TObject);
var
  node: TRedBlackTreeNode;
begin
  try
    node := strom.search(StrToInt((Sender as TEdit).Text));

    strom.highlight(node);

    Button1.Enabled := (node = nil);
    Button2.Enabled := (node <> nil);

   except
     on Exception : EConvertError do
       begin
         Button1.Enabled := False;
         Button2.Enabled := False;
         strom.highlight(NIL);
       end;
   end;
end;

procedure TFrmMainWindow.Edit1KeyPress(Sender: TObject; var Key: char);
begin
  if Key = #13 then
    begin
      if self.Button1.Enabled then
        self.Button1.Click;

      if self.Button2.Enabled then
        self.Button2.Click;
    end;
end;

procedure TFrmMainWindow.FormActivate(Sender: TObject);
begin
  if Edit1.Enabled then
    begin
      Edit1.SetFocus;
    end;

end;

procedure TFrmMainWindow.lblFontNodesClick(Sender: TObject);
begin
  self.fdNodes.Execute;
end;

procedure TFrmMainWindow.lblFontNilLeavesClick(Sender: TObject);
begin
  self.fdNilLeaves.Execute;
end;

procedure TFrmMainWindow.ScrollBar1Change(Sender: TObject);
begin
  strom.verticalSpacing := ScrollBar1.Position;
  Label3.Caption := IntToStr(strom.verticalSpacing);
end;

procedure TFrmMainWindow.ScrollBar2Change(Sender: TObject);
begin
  strom.horizontalSpacing := ScrollBar2.Position / 100;
  Label4.Caption := IntToStr(Round(strom.horizontalSpacing * 100)) + '%';
end;

procedure TFrmMainWindow.FormResize(Sender: TObject);
begin
  Image1.Picture.Graphic.Width := Image1.Width;
  Image1.Picture.Graphic.Height := Image1.Height;

  strom.draw;
end;

procedure TFrmMainWindow.ToolButton1Click(Sender: TObject);
begin
  strom.clear();
end;

procedure TFrmMainWindow.ToolButton2Click(Sender: TObject);
begin

  if odTree.Execute then
    begin
      strom.deserialize(odTree.FileName);
    end;

end;

procedure TFrmMainWindow.ToolButton3Click(Sender: TObject);
begin

  if self.sdTree.Execute then
    begin
      strom.serialize(sdTree.FileName);
    end;

end;

procedure TFrmMainWindow.btnUndoClick(Sender: TObject);
begin
  strom.previousSnapshot();
end;

procedure TFrmMainWindow.btnRedoClick(Sender: TObject);
begin
  strom.nextSnapshot();
end;

procedure TFrmMainWindow.updateInfo(Sender: TObject);
begin
  strom.draw();

  lblHeight.Caption := IntToStr(strom.height());
  lblWidth.Caption := IntToStr(strom.width());
  lblBlackHeight.Caption := IntToStr(strom.blackHeight());
  lblMinimum.Caption := IntToStr(strom.minimum());
  lblMaximum.Caption := IntToStr(strom.maximum());

  lblFontNodes.Caption := strom.fontForNodes.Name + ', ' + IntToStr(strom.fontForNodes.Size);
  lblFontNilLeaves.Caption := strom.fontForNilLeaves.Name + ', ' + IntToStr(strom.fontForNilLeaves.Size);

  btnUndo.Enabled := strom.hasPreviousSnapshot();
  btnRedo.Enabled := strom.hasNextSnapshot();
end;

procedure TFrmMainWindow.handleInput(Sender: TObject);
begin

  if ((Sender as TButton).Tag = 1) then
    begin
      strom.insert(StrToInt(Edit1.Text));
    end;

  if ((Sender as TButton).Tag = 2) then
    begin
      ShowMessage('Vymazávanie padá - nie je povolené. :(');
      //strom.delete(StrToInt(Edit1.Text));
    end;

  newElement();

end;

procedure handleNode(node: TRedBlackTreeNode);
begin
  FrmInfoWindow.Memo1.Text := FrmInfoWindow.Memo1.Text + IntToStr(node.key) + ' ';
end;

procedure TFrmMainWindow.traverseTree(Sender: TObject);
begin

  FrmInfoWindow.Memo1.Clear;

  case ((Sender as TButton).Tag) of
    1:
    begin
         strom.preorder(@handleNode);
         FrmInfoWindow.Caption := 'Pre-order';
    end;
    2:
    begin
         strom.inorder(@handleNode);
         FrmInfoWindow.Caption := 'In-order';
    end;
    3:
    begin
         strom.postorder(@handleNode);
         FrmInfoWindow.Caption := 'Post-order';
    end;
    4:
    begin
         strom.levelorder(@handleNode);
         FrmInfoWindow.Caption := 'Level-order';
    end;
  end;

  FrmInfoWindow.Show;
end;

initialization
  {$I uMain.lrs}
  {$I uInfo.lrs}

end.
