unit uRedBlackTreeNode;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Graphics, TypInfo;

type
  TRedBlackTreeNodeColor = (Red, Black);

  TRedBlackTreeNode = class
    _key: Integer;
    _color: TRedBlackTreeNodeColor;
    _left, _right: TRedBlackTreeNode;
    _parent: TRedBlackTreeNode;

    constructor Create(key: Integer; color: TRedBlackTreeNodeColor; left, right, parent: TRedBlackTreeNode);
    constructor Create(key: Integer; color: TRedBlackTreeNodeColor; parent: TRedBlackTreeNode);
    destructor Destroy(); override;

    function getRGB():Graphics.TColor;
    procedure setColor(color: TRedBlackTreeNodeColor);

    property key: Integer
      read _key
      write _key;

    property color: TRedBlackTreeNodeColor
      read _color
      write setColor;

    property parent: TRedBlackTreeNode
      read _parent
      write _parent;

    property left: TRedBlackTreeNode
      read _left
      write _left;

    property right: TRedBlackTreeNode
      read _right
      write _right;
  end;

implementation

constructor TRedBlackTreeNode.Create(key: Integer; color: TRedBlackTreeNodeColor; left, right, parent: TRedBlackTreeNode);
begin
  self.key := key;
  self.color := color;
  self.left := left;
  self.right := right;
  self.parent := parent;
end;

constructor TRedBlackTreeNode.Create(key: Integer; color: TRedBlackTreeNodeColor; parent: TRedBlackTreeNode);
begin
  self.Create(key, color, nil, nil, parent);
end;

destructor TRedBlackTreeNode.Destroy;
begin

  if self.left <> NIL then
    begin
      self.left.Destroy();
    end;

  if self.right <> NIL then
    begin
      self.right.Destroy();
    end;

  if (self.parent <> NIL) then
    begin
      if (self.parent.left = self) then
        begin
          self.parent.left := NIL;
        end
      else
        begin
          self.parent.right := NIL;
        end;
    end;

  inherited;

end;

function TRedBlackTreeNode.getRGB():Graphics.TColor;
begin
  if self.color = Red then
    Result := Graphics.clRed
  else
    Result := Graphics.clBlack;
end;

procedure TRedBlackTreeNode.setColor(color: TRedBlackTreeNodeColor);
begin
  self._color := color;
end;

end.

